##############################################################################
#
# Copyright (c) 2001-2004 Zope Corporation and Contributors.
# Copyright (c) 2004 Christian Heimes and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL). A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""SMTP mail objects for Japanese  """
__version__ = "0.4.3"

from email.Header import Header
from Products.MailHost.MailHost import MailBase, MailHostError, _encode, _mungeHeaders
import email, rfc822, string, types
from cStringIO import StringIO
from DateTime import DateTime

debug=1
def tojis(s, enc, decode_header = False):
    """ Convert to 'iso-2022-jp' encoding from encoding enc. 
    """
    if decode_header:
        us = u''
        for senc in  email.Header.decode_header(s):
            hs, henc = senc
            if henc:
                us += unicode(hs, henc, 'replace')
            else:
                us += unicode(hs, enc, 'replace')
    else:
        us = unicode(s, enc, 'replace')
            
    return us.encode('iso-2022-jp', 'replace')


try:
    from Products.SecureMailHost.mail import Mail
    from Products.SecureMailHost.SecureMailHost import SecureMailBase
    def _send(self, mfrom, mto, messageText, debug=False):
        """Send the message instead of SecureMailBase._send()
        """
        if not isinstance(messageText, email.MIMEText.MIMEText):
            message = email.message_from_string(messageText)
        else:
            message = messageText
    
        if self.title and not message.has_key('X-Converted-By'):
            message['X-Converted-By'] = 'jaMailHost ' + __version__
            for h in ('Subject', 'From', 'To', 'Cc'):
                if message.has_key(h):
                    message.replace_header(h, _enc_jp_header(
                                          tojis(message[h], self.title, True)))
            mfrom =  _enc_jp_header(mfrom)
            if isinstance(mto, types.ListType):
                mto = [_enc_jp_header(s) for s in mto]
            else:
                mto = _enc_jp_header(mto)
        
            payload = message.get_payload(decode=True)
            if not isinstance(payload, types.ListType):
                del message['Content-Transfer-Encoding']
                message.set_payload(tojis(payload, self.title), 'iso-2022-jp')
    
        mail = Mail(mfrom, mto, message, smtp_host=self.smtp_host,
                    smtp_port=self.smtp_port, userid=self._smtp_userid,
                    password=self._smtp_pass
                   )
        if debug:
            return mail
        else:
            mail.send()
    
    SecureMailBase._send=_send
except:
    pass


def sendTemplate(trueself, self, messageTemplate,
                 statusTemplate=None, mto=None, mfrom=None,
                 encode=None, REQUEST=None):
    """render a mail template, then send it... 
       Instead of MailBase.sendTemplate() """
    mtemplate = getattr(self, messageTemplate)
    messageText = mtemplate(self, trueself.REQUEST)
    if trueself.title:
        messageText = tojis(messageText, trueself.title)
        if mto:
            mto=tojis(mto, trueself.title)
        if mfrom:
            mfrom=tojis(mfrom, trueself.title)
        messageText, mto, mfrom = _jp_mungeHeaders( messageText, mto, mfrom)
    else:
        messageText, mto, mfrom = _mungeHeaders( messageText, mto, mfrom)
    messageText=_encode(messageText, encode)
    self._send(mfrom, mto, messageText)

    if not statusTemplate: return "SEND OK"

    try:
        stemplate=getattr(self, statusTemplate)
        return stemplate(self, trueself.REQUEST)
    except:
        return "SEND OK"

MailBase.sendTemplate=sendTemplate


def send(self, messageText, mto=None, mfrom=None, subject=None,
         encode=None):
    """Send the message instead of MailBase.send()
    """
    if self.title:
        messageText = tojis(messageText, self.title)
        if mto:
            mto=tojis(mto, self.title)
        if mfrom:
            mfrom=tojis(mfrom, self.title)
        if subject:
            subject=tojis(subject, self.title)
        messageText, mto, mfrom = _jp_mungeHeaders( messageText, mto, mfrom, subject)
    else:
        messageText, mto, mfrom = _mungeHeaders( messageText, mto, mfrom, subject)
    messageText = _encode(messageText, encode)
    self._send(mfrom, mto, messageText)

MailBase.send=send


def _jp_mungeHeaders( messageText, mto=None, mfrom=None, subject=None):
    """Sets missing message headers, and deletes Bcc.
       returns fixed message, fixed mto and fixed mfrom"""
    mfile=StringIO(messageText.strip())
    mo=rfc822.Message(mfile)

    # Parameters given will *always* override headers in the messageText.
    # This is so that you can't override or add to subscribers by adding them to
    # the message text.
    if subject:
        mo['Subject'] = subject
    elif not mo.getheader('Subject'):
        mo['Subject'] = '[No Subject]'

    if mto:
        if isinstance(mto, types.StringType):
            mto=map(lambda x:x.strip(), mto.split(','))
        if not mo.getheader('To'):
            mo['To'] = ','.join(mto)
    else:
        if not mo.getheader('To'):
            raise MailHostError,"Message missing SMTP Header 'To'"
        mto = map(lambda x:x.strip(), mo['To'].split(','))
        if mo.getheader('Cc'):
            mto = mto + map(lambda x:x.strip(), mo['Cc'].split(','))
        if mo.getheader('Bcc'):
            mto = mto + map(lambda x:x.strip(), mo['Bcc'].split(','))

    if mfrom:
        mo['From'] = mfrom
    else:
        if mo.getheader('From') is None:
            raise MailHostError,"Message missing SMTP Header 'From'"
        mfrom = mo['From']

    if mo.getheader('Bcc'):
        mo.__delitem__('Bcc')

    if not mo.getheader('Date'):
        mo['Date'] = DateTime().rfc822()

    if not mo.getheader('Mime-Version'):
        mo['Mime-Version']='1.0'
    mo['Content-Type']='text/plain; charset=iso-2022-jp'

    if mo.getheader('Subject'):
        mo['Subject'] = _enc_jp_header(mo['Subject'])
    if mo.getheader('From'):
        mo['From'] =  _enc_jp_header(mo['From'])
    if mo.getheader('To'):
        mo['To'] =  _enc_jp_header(mo['To'])
    if mo.getheader('Cc'):
        mo['Cc'] =  _enc_jp_header(mo['Cc'])
    mfrom =  _enc_jp_header(mfrom)
    mto = [_enc_jp_header(s) for s in mto]

    mo['X-Converted-By'] = 'jaMailHost ' + __version__

    mo.rewindbody()
    finalmessage = mo
    finalmessage = mo.__str__() + '\n' + mfile.read()
    mfile.close()
    return finalmessage, mto, mfrom

def _enc_jp_header(src):
    if string.find(src, chr(033)) < 0:
        return src
    
    flist = string.split(src, chr(033))
    dest = flist[0]
    
    for f in flist[1:]:
        if f[:2] == r'$@' or f[:2] == r'$B':
            dest += str(Header(chr(033) + f, 'iso-2022-jp'))
        elif f[:2] == r'(B' or f[:2] == r'(J':
            dest += f[2:]
    return dest

